//
//  PTRGeofenceManagerInterface.h
//  Pointr
//
//  Created by Pointr Labs on 21/03/16.
//  Copyright © 2016 Pointr. All rights reserved.
//

#import <PointrKit/PTRBuilding.h>
#import <PointrKit/PTRListenerHelper.h>
#import <PointrKit/PTRSite.h>

NS_ASSUME_NONNULL_BEGIN

@class PTRGeofenceEvent;
@class PTRGeofence;
@class PTRPoi;
/**
 *  PTRGeofenceManager is responsible for users region enters and exits.
 */
OBJC_EXPORT @protocol PTRGeofenceManagerDelegate<NSObject>

@optional

/**
 *  Notifies listeners when a region is entered.
 *
 *  @param building The building which entered by the user.
 */
- (void)onGeofenceManagerDidEnterBuilding:(nonnull PTRBuilding *)building;

/**
 *  Notifies listeners when a region is exited.
 *
 *  @param building The building which exited by the user.
 */
- (void)onGeofenceManagerDidExitBuilding:(nonnull PTRBuilding *)building;

/**
 *  Notifies listeners when a geofence event is triggered.
 *
 *  @param event The event that is triggered.
 */
- (void)onGeofenceManagerGeofenceTriggered:(nonnull PTRGeofenceEvent *)event;

/**
 *  Notifies listeners when a poi trigger is entered.
 *
 *  @param triggerPoi The poi object that is triggered.
 */
- (void)onGeofenceManagerEnterTriggerPoi:(nonnull PTRPoi *)triggerPoi;

/**
 *  Notifies listeners when a poi trigger is exited.
 *
 *  @param triggerPoi The poi object that is triggered.
 */
- (void)onGeofenceManagerLeftTriggerPoi:(nonnull PTRPoi *)triggerPoi;

@end

/**
 *  Main manager class responsible for location related triggers (enter, exit)
 */
OBJC_EXPORT @protocol PTRGeofenceManagerInterface<NSObject>

PTR_GENERATE_LISTENER_DECLARATIONS(PTRGeofenceManagerDelegate)

/////////////////////////////////////////////////////////////////////////
#pragma mark - Public properties
/////////////////////////////////////////////////////////////////////////

/// Get the building the user is in.
/// @return `PTRBuilding` object.
- (nullable PTRBuilding *)currentBuilding;

/// Get the site the user is in.
/// @return `PTRSite` object.
- (nullable PTRSite *)currentSite;

/**
 * Get list of geofences that are in "inside" state
 * @return list of geofences that we are "inside"
 */
- (nonnull NSArray<PTRGeofence *> *)getInsideGeofences;

/**
 * Get list of all global geofences defined on Pointr Cloud
 * @return list of global geofences that are defined on Pointr Cloud
 */
- (nonnull NSArray<PTRGeofence *> *)getAllGlobalGeofences;

/**
 * Get the external trigger pois of the given site
 * @param site the site to get the external triggers of
 * @return a list of the external trigger pois of the site
 */
- (nonnull NSArray<PTRPoi *> *)getTriggerPoisForSite:(PTRSite *)site;

@end

NS_ASSUME_NONNULL_END
